@@ -23,6 +23,7 @@ import warnings
 import six
 
 from google.api_core import page_iterator
+from google.api_core import datetime_helpers
 from google.cloud._helpers import _datetime_to_rfc3339
 from google.cloud._helpers import _NOW
 from google.cloud._helpers import _rfc3339_to_datetime
@@ -40,6 +41,12 @@ from google.cloud.storage.notification import BucketNotification
 from google.cloud.storage.notification import NONE_PAYLOAD_FORMAT
 
 
+_LOCATION_SETTER_MESSAGE = (
+    "Assignment to 'Bucket.location' is deprecated, as it is only "
+    "valid before the bucket is created. Instead, pass the location "
+    "to `Bucket.create`.")
+
+
 def _blobs_page_start(iterator, page, response):
     """Grab prefixes after a :class:`~google.cloud.iterator.Page` started.
 
@@ -99,6 +106,170 @@ def _item_to_notification(iterator, item):
     return BucketNotification.from_api_repr(item, bucket=iterator.bucket)
 
 
+class LifecycleRuleConditions(dict):
+    """Map a single lifecycle rule for a bucket.
+
+    See: https://cloud.google.com/storage/docs/lifecycle
+
+    :type age: int
+    :param age: (optional) apply rule action to items whos age, in days,
+                exceeds this value.
+
+    :type created_before: datetime.date
+    :param created_before: (optional) apply rule action to items created
+                           before this date.
+
+    :type is_live: bool
+    :param is_live: (optional) if true, apply rule action to non-versioned
+                    items, or to items with no newer versions. If false, apply
+                    rule action to versioned items with at least one newer
+                    version.
+
+    :type matches_storage_class: list(str), one or more of
+                                 :attr:`Bucket._STORAGE_CLASSES`.
+    :param matches_storage_class: (optional) apply rule action to items which
+                                  whose storage class matches this value.
+
+    :type number_of_newer_versions: int
+    :param number_of_newer_versions: (optional) apply rule action to versioned
+                                     items having N newer versions.
+
+    :raises ValueError: if no arguments are passed.
+    """
+    def __init__(self, age=None, created_before=None, is_live=None,
+                 matches_storage_class=None, number_of_newer_versions=None,
+                 _factory=False):
+        conditions = {}
+
+        if age is not None:
+            conditions['age'] = age
+
+        if created_before is not None:
+            conditions['createdBefore'] = created_before.isoformat()
+
+        if is_live is not None:
+            conditions['isLive'] = is_live
+
+        if matches_storage_class is not None:
+            conditions['matchesStorageClass'] = matches_storage_class
+
+        if number_of_newer_versions is not None:
+            conditions['numNewerVersions'] = number_of_newer_versions
+
+        if not _factory and not conditions:
+            raise ValueError("Supply at least one condition")
+
+        super(LifecycleRuleConditions, self).__init__(conditions)
+
+    @classmethod
+    def from_api_repr(cls, resource):
+        """Factory:  construct instance from resource.
+
+        :type resource: dict
+        :param resource: mapping as returned from API call.
+
+        :rtype: :class:`LifecycleRuleConditions`
+        :returns: Instance created from resource.
+        """
+        instance = cls(_factory=True)
+        instance.update(resource)
+        return instance
+
+    @property
+    def age(self):
+        """Conditon's age value."""
+        return self.get('age')
+
+    @property
+    def created_before(self):
+        """Conditon's created_before value."""
+        before = self.get('createdBefore')
+        if before is not None:
+            return datetime_helpers.from_iso8601_date(before)
+
+    @property
+    def is_live(self):
+        """Conditon's 'is_live' value."""
+        return self.get('isLive')
+
+    @property
+    def matches_storage_class(self):
+        """Conditon's 'matches_storage_class' value."""
+        return self.get('matchesStorageClass')
+
+    @property
+    def number_of_newer_versions(self):
+        """Conditon's 'number_of_newer_versions' value."""
+        return self.get('numNewerVersions')
+
+
+class LifecycleRuleDelete(dict):
+    """Map a lifecycle rule deleting matching items.
+
+    :type kw: dict
+    :params kw: arguments passed to :class:`LifecycleRuleConditions`.
+    """
+    def __init__(self, **kw):
+        conditions = LifecycleRuleConditions(**kw)
+        rule = {
+            'action': {
+                'type': 'Delete',
+            },
+            'condition': dict(conditions),
+        }
+        super(LifecycleRuleDelete, self).__init__(rule)
+
+    @classmethod
+    def from_api_repr(cls, resource):
+        """Factory:  construct instance from resource.
+
+        :type resource: dict
+        :param resource: mapping as returned from API call.
+
+        :rtype: :class:`LifecycleRuleDelete`
+        :returns: Instance created from resource.
+        """
+        instance = cls(_factory=True)
+        instance.update(resource)
+        return instance
+
+
+class LifecycleRuleSetStorageClass(dict):
+    """Map a lifecycle rule upating storage class of matching items.
+
+    :type storage_class: str, one of :attr:`Bucket._STORAGE_CLASSES`.
+    :param storage_class: new storage class to assign to matching items.
+
+    :type kw: dict
+    :params kw: arguments passed to :class:`LifecycleRuleConditions`.
+    """
+    def __init__(self, storage_class, **kw):
+        conditions = LifecycleRuleConditions(**kw)
+        rule = {
+            'action': {
+                'type': 'SetStorageClass',
+                'storageClass': storage_class,
+            },
+            'condition': dict(conditions),
+        }
+        super(LifecycleRuleSetStorageClass, self).__init__(rule)
+
+    @classmethod
+    def from_api_repr(cls, resource):
+        """Factory:  construct instance from resource.
+
+        :type resource: dict
+        :param resource: mapping as returned from API call.
+
+        :rtype: :class:`LifecycleRuleDelete`
+        :returns: Instance created from resource.
+        """
+        action = resource['action']
+        instance = cls(action['storageClass'], _factory=True)
+        instance.update(resource)
+        return instance
+
+
 class Bucket(_PropertyMixin):
     """A class representing a Bucket on Cloud Storage.
 
@@ -927,11 +1098,18 @@ class Bucket(_PropertyMixin):
         :setter: Set lifestyle rules for this bucket.
         :getter: Gets the lifestyle rules for this bucket.
 
-        :rtype: list(dict)
+        :rtype: generator(dict)
         :returns: A sequence of mappings describing each lifecycle rule.
         """
         info = self._properties.get('lifecycle', {})
-        return [copy.deepcopy(rule) for rule in info.get('rule', ())]
+        for rule in info.get('rule', ()):
+            action_type = rule['action']['type']
+            if action_type == 'Delete':
+                yield LifecycleRuleDelete.from_api_repr(rule)
+            elif action_type == 'SetStorageClass':
+                yield LifecycleRuleSetStorageClass.from_api_repr(rule)
+            else:
+                raise ValueError("Unknown lifecycle rule: {}".format(rule))
 
     @lifecycle_rules.setter
     def lifecycle_rules(self, rules):
@@ -943,8 +1121,54 @@ class Bucket(_PropertyMixin):
         :type entries: list of dictionaries
         :param entries: A sequence of mappings describing each lifecycle rule.
         """
+        rules = [dict(rule) for rule in rules]  # Convert helpers if needed
         self._patch_property('lifecycle', {'rule': rules})
 
+    def clear_lifecyle_rules(self):
+        """Set lifestyle rules configured for this bucket.
+
+        See https://cloud.google.com/storage/docs/lifecycle and
+             https://cloud.google.com/storage/docs/json_api/v1/buckets
+        """
+        self.lifecycle_rules = []
+
+    def add_lifecycle_delete_rule(self, **kw):
+        """Add a "delete" rule to lifestyle rules configured for this bucket.
+
+        See https://cloud.google.com/storage/docs/lifecycle and
+             https://cloud.google.com/storage/docs/json_api/v1/buckets
+
+        .. literalinclude:: snippets.py
+          :start-after: [START add_lifecycle_delete_rule]
+          :end-before: [END add_lifecycle_delete_rule]
+
+        :type kw: dict
+        :params kw: arguments passed to :class:`LifecycleRuleConditions`.
+        """
+        rules = list(self.lifecycle_rules)
+        rules.append(LifecycleRuleDelete(**kw))
+        self.lifecycle_rules = rules
+
+    def add_lifecycle_set_storage_class_rule(self, storage_class, **kw):
+        """Add a "delete" rule to lifestyle rules configured for this bucket.
+
+        See https://cloud.google.com/storage/docs/lifecycle and
+             https://cloud.google.com/storage/docs/json_api/v1/buckets
+
+        .. literalinclude:: snippets.py
+          :start-after: [START add_lifecycle_set_storage_class_rule]
+          :end-before: [END add_lifecycle_set_storage_class_rule]
+
+        :type storage_class: str, one of :attr:`_STORAGE_CLASSES`.
+        :param storage_class: new storage class to assign to matching items.
+
+        :type kw: dict
+        :params kw: arguments passed to :class:`LifecycleRuleConditions`.
+        """
+        rules = list(self.lifecycle_rules)
+        rules.append(LifecycleRuleSetStorageClass(storage_class, **kw))
+        self.lifecycle_rules = rules
+
     _location = _scalar_property('location')
 
     @property
@@ -976,10 +1200,7 @@ class Bucket(_PropertyMixin):
             to `Bucket.create`.
         """
         warnings.warn(
-            "Assignment to 'Bucket.location' is deprecated, as it is only "
-            "valid before the bucket is created. Instead, pass the location "
-            "to `Bucket.create`.",
-            DeprecationWarning)
+            _LOCATION_SETTER_MESSAGE, DeprecationWarning, stacklevel=2)
         self._location = value
 
     def get_logging(self):
